/*

The Life of a Geek
Copyright (C) 2003-2004 Daniel Quintiliani

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

You can e-mail Daniel Quintiliani at coredump0@mercurylink.net

3/16/04: Daniel Quintiliani <coredump0@mercurylink.net>: FixTime() fixed, fixed
GetTotalDays().
2/3/04: Daniel Quintiliani <coredump0@mercurylink.net>: Updated e-mail address,
added GetTotalDays(), updated DaysInMonth(), updated copyright.
7/9/03: Daniel Quintiliani <coredump@coredumpcentral.org>: Updated 
DaysInMonth().
7/4/03: Daniel Quintiliani <coredump@coredumpcentral.org>: Added AddYear(),
now a real Gregorian calendar.
eltime.cc: Elapsed time

*/

#include "version.h"

#if defined _ISOC9X_SOURCE && defined _ISOC99_SOURCE
#include <math.h>
#else
#include "strlwr.h"
#endif

#include "eltime.h"

ElTime::ElTime()
{
	Year = 0;
	Month = 0;
	MDay = 0;
	Hour = 0;
	Min = 0;
}

ElTime::ElTime(ElTime& elt)
{
	Year = elt.GetYear();
	Month = elt.GetMonth();
	MDay = elt.GetMDay();
	Hour = elt.GetHour();
	Min = elt.GetMin();
}

ElTime::~ElTime() { }

int ElTime::GetYear() { return Year; }
int ElTime::GetMonth() { return Month; }
int ElTime::GetMDay() { return MDay; }
int ElTime::GetHour() { return Hour; }
int ElTime::GetMin() { return Min; }

void ElTime::AddYear(int n)
{
	Year += n;
}

void ElTime::AddMonth(int n)
{
	Month += n;
	FixTime();
}

void ElTime::AddMDay(int n)
{
	MDay += n;
	FixTime();
}

void ElTime::AddHour(int n)
{
	Hour += n;
	FixTime();
}

void ElTime::AddMin(int n)
{
	Min += n;
	FixTime();
}

void ElTime::FixTime()
{
	while(Min >= 60)
	{
		Min -= 60;
		Hour++;
	}

	while(Hour >= 24)
	{
		Hour -= 24;
		MDay++;
	}
		
	while(MDay >= DaysInMonth(20))
	{
		MDay -= DaysInMonth(20);
		Month++;
	}

	while(Month >= 12)
	{
		Month -= 12;
		Year++;
	}
}

int ElTime::DaysInMonth(int n)
{
	int MonthUsed;
	if(n==20) MonthUsed = Month; else MonthUsed = n;
	// Everything is one less, this is elapsed time, not a calendar
	if(MonthUsed == 8 || MonthUsed == 3 || MonthUsed == 5 || MonthUsed == 10)
		return 30;
	else if(MonthUsed == 1)
	{
		if(Year == 0 || (Year / 4) == 0)
			return 29;
		else
			return 28;
	}
	else
		return 31;
}

int ElTime::GetTotalDays()
{
	int TotalDays = (int)roundf((Year * 365.25));
	for(int n = 0; n < Month; n++)
		TotalDays += DaysInMonth(n);
	TotalDays += (int)roundf((Hour / 24) + (Min / 1440));
	return TotalDays;
}
